package ru.ulstu.extractor.gitrepository.service;

import org.springframework.stereotype.Service;
import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.http.HttpService;
import ru.ulstu.extractor.util.StringUtils;

@Service
public class GithubApi implements GitApi {
    private final HttpService httpService;
    private static final String BRANCHES_COUNT_URL = "https://api.github.com/repos/%s/%s/branches";
    private static final String STARS_COUNT_URL = "https://api.github.com/repos/%s/%s";
    private static final String OPEN_ISSUES_URL = "https://api.github.com/repos/%s/%s/issues?state=open";
    private static final String AUTHOR_COMPLETED_ISSUES_URL = "https://api.github.com/repos/%s/%s/issues?state=open";

    public GithubApi(HttpService httpService) {
        this.httpService = httpService;
    }

    @Override
    public Integer getBranchesCount(Branch branch) {
        return httpService
                .get(getFormattedUrl(branch, BRANCHES_COUNT_URL))
                .length();
    }

    @Override
    public Integer getStarsCount(Branch branch) {
        return httpService
                .get(getFormattedUrl(branch, STARS_COUNT_URL))
                .getJSONObject(0)
                .getInt("watchers");
    }

    @Override
    public Integer getOpenIssuesCount(Branch branch) {
        return httpService
                .get(getFormattedUrl(branch, OPEN_ISSUES_URL))
                .length();
    }

    @Override
    public String getFormattedUrl(Branch branch, String template) {
        String[] urlParts = branch.getGitRepository().getUrl().split("/");
        return String.format(template, urlParts[3], StringUtils.removeDotGit(urlParts[4]));
    }

    @Override
    public Integer getAuthorsCompletedIssues(Branch branch) {
        return httpService
                .get(getFormattedUrl(branch, AUTHOR_COMPLETED_ISSUES_URL))
                .length();
    }

    @Override
    public String getRepositoryId(String repositoryUrl) {
        return null;
    }
}
