package ru.ulstu.extractor.http;

import org.json.JSONArray;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;

import java.time.Duration;
import java.util.Optional;

@Service
public class HttpService {
    private final Logger log = LoggerFactory.getLogger(HttpService.class);
    private final WebClient client;

    public HttpService(WebClient client) {
        this.client = client;
    }

    public JSONObject post(String url, JSONObject postData) {
        log.debug("Service call: {}", url);
        JSONObject response = null;
        try {
            response = new JSONObject(Optional.ofNullable(client
                    .post()
                    .uri(url)
                    .contentType(MediaType.APPLICATION_JSON)
                    .body(BodyInserters.fromValue(postData.toString()))
                    .accept(MediaType.APPLICATION_JSON)
                    .retrieve()
                    .bodyToMono(String.class)
                    .toFuture().get()).orElse("{response:\"empty\"}"));
        } catch (Exception e) {
            return new JSONObject("{response:\"empty\"}");
        }
        log.debug("Service response: {}", response);
        return response;
    }

    public JSONArray get(String url) {
        log.debug("Service call: {}", url);
        try {
            String response = client
                    .get()
                    .uri(url)
                    .accept(MediaType.APPLICATION_JSON)
                    .retrieve()
                    .bodyToMono(String.class)
                    .timeout(Duration.ofMinutes(1))
                    .toFuture().get();
            if (response.startsWith("[")) {
                return new JSONArray(response);
            } else {
                JSONArray jsonArray = new JSONArray();
                jsonArray.put(0, new JSONObject(response));
                return jsonArray;
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
